PROG_DINOR1   .MACRO
;****************************************************
;+   download module for flash DINOR type           +
;****************************************************
load_prog:
;****************************************************
;+  initialize                                      +
;****************************************************
  LDC   #ISP_ADDR,ISP           ; isp reset
  MOV.W #(load_receive-load_prog+ram_prog),A0 ; UART1 interrupt address
.if UART==0
  MOV.W A0,F_INTB+18*4          ; UART0 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+18*4+2      ; UART0 interrupt vector address(H)
.endif
.if UART==1
  MOV.W A0,F_INTB+20*4          ; UART1 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+20*4+2      ; UART1 interrupt vector address(H)
.endif
.if UART==2
  MOV.W A0,F_INTB+16*4          ; UART2 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+16*4+2      ; UART2 interrupt vector address(H)
.endif
  LDINTB #F_INTB
  MOV.B #00000011B,PRCR         ; PRC0/PRC1 protect off ** chg **
  MOV.B #00000000B,CM0          ; main clock CM16,CM17 set ** chg **
  MOV.B #01100000B,CM1          ; 1/2 cycle ** chg **
  MOV.B #10001000B,PM1          ; 1 wait & RAM 23K EXT
  MOV.B #00000000B,PRCR         ; protect on ** chg **
  MOV.B #00000000B,FMCR         ; flash memory control register
  MOV.B #00000010B,FMCR         ; flash memory control register(CPU Write mode)
  MOV.B #00000110B,FMCR         ; flash memory control register(rock bit off)
  FSET  I

;****************************************************
;+  block erase                                     +
;****************************************************
block_erase:
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase             ; C = 0 jump block_erase
  BCLR  1,FMCR                  ; normal mode
  BSET  1,FMCR                  ; CPU write mode
  MOV.W #FLASH_START,A1         ; block6-3 top address set
block_erase_1:                  ; erase block 6-3(0C0000h-0F0000h)
  MOV.W #0000H,A0               ; 0000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0xFFFEh block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0xFFFEh block erase verify
block_erase_2:
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_2           ; BUSY jump block_erase_2
  ADD.W #1,A1                   ; NEXT block top address
  CMP.W #10H,A1                 ; block3 top address ?
  JNZ   block_erase_1           ; block erase 0F0000h no complete jump block_erase_1
block_erase_3:                  ; erase block 2(0F8000h)
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_3           ; BUSY jump block_erase_3
  MOV.W #0FH,A1                 ; block-2 top address set
  MOV.W #8000H,A0               ; 008000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0F8000h block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0F8000h block erase verify
block_erase_4:                  ; erase block 2(0FA000h)
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_4           ; BUSY jump block_erase_4
  MOV.W #0FH,A1                 ; block-2 top address set
  MOV.W #0A000H,A0              ; 00A000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0FA000h block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0FA000h block erase verify
block_erase_5:
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_5

;****************************************************
;+   transmit 1                                     +
;****************************************************
  MOV.W #0,R0                   ; R0 data clear
  MOV.B #0,mode                 ; mode data clear
  MOV.B #0,p_mode               ; p_mode data clear
  MOV.B #0,ring_numb            ; ring_numb data clear
  MOV.B #0,str_numb             ; str_numb data clear
  MOV.B #0,read_numb            ; read_numb data clear
  MOV.W #0,A1                   ; A1 data clear
  MOV.B #28H,UxTB               ; START code send
transmit1_start_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_start_check
  MOV.B #30H,UxTB               ; NORMAL code send
transmit1_normal_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_normal_check
  MOV.B #29H,UxTB               ; END code send
transmit1_end_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_end_check

;****************************************************
;+   program                                        +
;****************************************************
program:
  CMP.B ring_numb,read_numb     ; download interrupt data number check
  JEQ   program
  MOV.B read_numb,A0            ; read_numb set
  MOV.B dat_buff[A0],R0L       ; dat_buff data 1byte get
  ADD.B #1,str_numb             ; str_numb +1
  ADD.B #1,read_numb            ; read_numb +1
mode_check:
  CMP.B #0,mode                 ; mode check
  JEQ   start_check             ; start code check
  CMP.B #1,mode                 ; mode check
  JEQ   address_set             ; start address check
  CMP.B #2,mode                 ; mode check
  JEQ   length_set              ; length data check
  CMP.B #3,mode                 ; mode check
  JEQ   data_program            ; bin data check
  CMP.B #4,mode                 ; mode check
  JEQ   end_check               ; end code check
  JMP   program

;****************************************************
;+   start check                                    +
;****************************************************
start_check:
  MOV.B str_numb,A0             ; str_numb set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #3,str_numb             ; receive top data 3byte check
  JLTU  start_check_end
  MOV.B #0,info+3
  CMP.B #28h,info               ; 2nd Start code ?
  JNZ   start_check_end
  CMP.B #30h,info+1             ; 2nd DWNH code ?
  JNZ   start_check_end
  CMP.B #42h,info+2             ; 2nd DWNH code ?
  JNZ   start_check_end
  MOV.B #1,mode                 ; address_set mode set
  MOV.W #0,str_numb             ; stock data clear
start_check_end:
  JMP   program

;****************************************************
;+   address set                                    +
;****************************************************
address_set:
  MOV.W #0,prog_addr            ; prog_address clear
  MOV.W #0,prog_addr+2          ; prog_address clear
  MOV.B str_numb,A0             ; str_numb set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #6,str_numb             ; start address data 6byte check
  JLTU  address_set_end
  MOV.W #info,A0                ; high address get
  MOV.B #6,R1L                  ; data set
  JSR   ascBinCnv
  MOV.W #info,A0
  JSR   addrCnv
  MOV.W tmp_addr,prog_addr      ; tmp_adress set
  MOV.W tmp_addr+2,prog_addr+2  ; tmp_address set
  MOV.B #2,mode                 ; length mode set
  MOV.B #0,str_numb             ; stock data clear
address_set_end:
  JMP   program

;****************************************************
;+   length set                                     +
;****************************************************
length_set:
  MOV.W #0,dat_length           ; dat_length buffer clear
  MOV.B str_numb,A0             ; str count set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #4,str_numb             ; length data 4byte check
  JLTU  length_set_end
  MOV.W #info,A0
  MOV.B #4,R1L
  JSR   ascBinCnv
  MOV.W #info-2,A0
  JSR   addrCnv
  MOV.W tmp_addr,dat_length     ; tmp_adress set
  MOV.B #3,mode                 ; data program mode set
  MOV.B #0,str_numb             ; stock data clear
length_set_end:
  JMP   program

;****************************************************
;+   data program                                   +
;****************************************************
data_program:
  CMP.B #0,p_mode
  JNE   data_program_1
  MOV.B #2,p_mode               ; ff_set mode set
  MOV.W prog_addr,page_addr     ; program address set
  MOV.W prog_addr+2,page_addr+2 ; program address set
data_program_1:
  CMP.B #1,p_mode               ; 1 page write mode ?
  JNE   data_program_3
  CMP.W prog_addr+1,page_addr+1
  JNE   data_program_2
  MOV.B #3,p_mode               ; ESC code check mode
  MOV.B prog_addr,page_addr     ; program address set
  JMP   data_program_4
data_program_2:
  MOV.B #2,p_mode               ; ff_set mode set
  JSR   page_program
  MOV.W prog_addr,page_addr     ; program address set
  MOV.W prog_addr+2,page_addr+2 ; program address set
data_program_3:
  CMP.B #2,p_mode               ; ff_set mode ?
  JNE   data_program_4
  MOV.B #3,p_mode               ; ESC code check mode
  JSR   ff_set
data_program_4:
  CMP.B #1BH,R0L                ; ESC code ?
  JNE   data_program_5
  MOV.B #40H,R0H                ; Bin data ?
  JMP   data_program_end
data_program_5:
  SUB.B R0H,R0L                 ; data get
  MOV.B #0,R0H                  ; data high 8bit clear
  MOV.B page_addr,A0            ; page address set
  MOV.B R0L,data[A0]            ; data set
  SUB.W #1,dat_length           ; data length -1
  CMP.W #0,dat_length           ; data length 0 ?
  JNE   data_program_6
  MOV.B #4,mode                 ; end_check mode set
  MOV.B #1,p_mode               ; 1 page write mode 
  MOV.B #0,str_numb             ; stock data clear
  JMP   data_program_end
data_program_6:
  CMP.B #0FFH,page_addr         ; page address END ?
  JNE   data_program_7
  JSR   page_program            ; page write go
  MOV.B #2,p_mode               ; buffer FF write mode 
data_program_7:
  ADD.W #1,page_addr            ; next page address set
  ADCF.B page_addr+2            ; next page address set
data_program_end:
  JMP   program

;****************************************************
;+   end check                                      +
;****************************************************
end_check:
  CMP.B #3,str_numb             ; receive end data 3byte check
  JNE   end_check_end
  CMP.B #2EH,R0L                ; RCVE command check
  JEQ   end_sequence
  MOV.B #1,mode                 ; address set mode
  MOV.B #1,str_numb             ; stock data 1
  JMP   address_set
end_check_end:
  JMP   program
end_sequence:
  JSR   page_program

;****************************************************
;+   transmit 2                                     +
;****************************************************
  MOV.B #28H,UxTB               ; START code send
transmit2_start_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_start_check
  MOV.B #30H,UxTB               ; NORMAL code send
transmit2_normal_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_normal_check
  MOV.B #29H,UxTB               ; END code send
transmit2_end_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_end_check
transmit2_send_check:
  BTST  3,UxC0                  ; data send check
  JZ    transmit2_send_check

;****************************************************
;+   software reset                                 +
;****************************************************
  MOV.B #00000011,PRCR          ; protect off (2002.03.27 changed)
  MOV.W #0AAAAH,brg_chk_flg     ; auto baud rate set ok flag
  MOV.B #01001000B,CM0          ; clock 1/8 (2002.03.27 changed)
  MOV.B #00001000B,PM0          ; software reset
  NOP
  NOP
  NOP
  NOP
  NOP
  NOP

;****************************************************
;+   ff_set soubroutine                             +
;****************************************************
ff_set:
  MOV.W #0,A0                   ; data clear
  MOV.W #80H,R2                 ; write count
ff_set_loop:
  MOV.W #0FFFFH,data[A0]        ; 0FFFFh data write
  ADD.W #2,A0                   ; next 1byte area
  SBJNZ.W  #1,R2,ff_set_loop    ; write count end?
  RTS

;****************************************************
;+   page program soubroutine                       +
;****************************************************
page_program:
  MOV.W #0,R3                   ; data clear
  MOV.W page_addr,A0            ; page addr middle address
  MOV.W page_addr+2,R3          ; page addr high address
  AND.W #0FF00H,A0              ; page addr middle addres mask
  MOV.W R3,A1                   ; page addr high address
page_program_internal:
  CMP.B #0FH,A1                 ; 0C0000-0E0000 area ?
  JLTU  page_program_write      ; small area => page program OK
  CMP.W #0BE00H,A0              ; USER vector move area ?
  JLTU  page_program_write      ; small area => page program OK
  CMP.W #0FE00H,A0              ; monitor vector area ?
  JLTU  page_program_end        ; small area => page program NG
  SUB.W #4000H,A0               ; USER vector area change move
page_program_write:
  MOV.W #80H,R2                 ; 256byte write count
  MOV.W #41H,R1                 ; page program command set
  STE.W R1,[A1A0]               ; page program start
  MOV.W #0,A1                   ; data clear
page_program_loop:
  MOV.W data[A1],R1             ; page program data set
  PUSH.W A1                     ; A0 data count shelter
  MOV.W R3,A1                   ; page address high 4bit again
  STE.W R1,[A1A0]               ; data page program write
  POP.W A1                      ; A0 data return
  ADD.W #2,A0                   ; page program address next 2byte
  ADD.B #2,A1                   ; next data count set
  SBJNZ.W #1,R2,page_program_loop
page_write_check:
  BTST  0,FMCR                  ; page write end ?
  JZ    page_write_check
page_program_end:
  RTS

;****************************************************
;+   load receive interrupt                         +
;****************************************************
load_receive:
  FSET  B                       ; BANK 1 register use
  MOV.B ring_numb,A0            ; ring_numb set
  MOV.B UxRB,dat_buff[A0]      ; load data set
  ADD.B #1,ring_numb            ; ring_numb +1
  FCLR  B                       ; BANK 0 change
  REIT

;----------------------------------------------------
;+  Address convert                                 +
;+     arg : a0  start address                      +
;+    rtn : tmp_addr                                +
;----------------------------------------------------
addrCnv:
  MOVLL 5[A0],R0L               ; low address low 4bit
  MOVLH 4[A0],R0L               ; low address high 4bit
  MOV.B R0L,tmp_addr            ; low address set
  MOVLL 3[A0],R0L               ; middle address low 4bit
  MOVLH 2[A0],R0L               ; middle address high 4bit
  MOV.B R0L,tmp_addr+1          ; middle set
  MOVLL 1[A0],R0L               ; high address low 4bit
  MOVLH [A0],R0L                ; high address high 4bit
  MOV.B R0L,tmp_addr+2          ; high address set
  MOV.B #0,tmp_addr+3           ; high high address ZERO set
  RTS

;----------------------------------------------------
;+  Address convert 5bytes                          +
;+     arg : a0  start address                      +
;+    rtn : tmp_addr                                +
;----------------------------------------------------
addrCnv5:
  MOVLL 4[A0],R0L               ; low address low 4bit
  MOVLH 3[A0],R0L               ; low address high 4bit
  MOV.B R0L,tmp_addr            ; low address set
  MOVLL 2[A0],R0L               ; middle address low 4bit
  MOVLH 1[A0],R0L               ; middle address high 4bit
  MOV.B R0L,tmp_addr+1          ; middle set
  XOR.B R0L,R0L
  MOVLL [A0],R0L                ; high address
  MOV.B R0L,tmp_addr+2          ; high address set
  MOV.B #0,tmp_addr+3
  RTS

;----------------------------------------------------
;+  Asc --> Bin convert                             +
;+    arg1 : a0  start address                      +
;+    arg2 : r1l convert times                      +
;+    rtn  : -                                      +
;----------------------------------------------------
ascBinCnv:
  MOV.B [A0],R0L
  CMP.B #40H,R0L
  JC    ascBinCnv_chk
  SUB.B #30H,R0L
  JMP   ascBin_next
ascBinCnv_chk:
  SUB.B #37H,R0L
ascBin_next:
  MOV.B R0L,[A0]
  ADD.W #1,A0
  SBJNZ.B #1,R1L,ascBinCnv
  RTS

;----------------------------------------------------
;+   download module for flash                      +
;----------------------------------------------------
  NOP
  NOP
  NOP
load_prog_end:

;****************************************************
;+   end download module for flash DINOR type       +
;****************************************************
  .ENDM
  
PROG_HND1  .MACRO
;****************************************************
;+   download module for flash HND type 1           +
;****************************************************
load_prog:
;****************************************************
;+  initialize                                      +
;****************************************************
  LDC   #ISP_ADDR,ISP           ; isp reset
  MOV.W #(load_receive-load_prog+ram_prog),A0 ; UART1 interrupt address
.if UART==0
  MOV.W A0,F_INTB+18*4          ; UART0 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+18*4+2      ; UART0 interrupt vector address(H)
.endif
.if UART==1
  MOV.W A0,F_INTB+20*4          ; UART1 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+20*4+2      ; UART1 interrupt vector address(H)
.endif
.if UART==2
  MOV.W A0,F_INTB+16*4          ; UART1 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+16*4+2      ; UART1 interrupt vector address(H)
.endif
  LDINTB #F_INTB
  MOV.B #00000011B,PRCR         ; PRC0/PRC1 protect off ** chg **
  MOV.B #00000000B,CM0          ; main clock CM16,CM17 set ** chg **
  MOV.B #01100000B,CM1          ; 1/2 cycle ** chg **
  MOV.B #10001000B,PM1          ; 1 wait & RAM 23K EXT
  MOV.B #00000000B,PRCR         ; protect on ** chg **
  MOV.B #00000000B,FMCR         ; flash memory control register
  MOV.B #00000010B,FMCR         ; flash memory control register(CPU Write mode)
  MOV.B #00000110B,FMCR         ; flash memory control register(lock bit off)
  FSET  I

;****************************************************
;+  block erase                                     +
;****************************************************
block_erase:
.if DEBUG==1
  MOV.B #03Fh,FMCR
.endif
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase             ; C = 0 jump block_erase
  BCLR  1,FMCR                  ; normal mode
  BSET  1,FMCR                  ; CPU write mode
  MOV.W #FLASH_START,A1         ; block6-3 top address set
block_erase_1:                  ; erase block 6-5(0C0000h-0F0000h)
  MOV.W #0000H,A0               ; 0000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0xFFFEh block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0xFFFEh block erase verify
block_erase_2:
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_2           ; BUSY jump block_erase_2
  ADD.W #1,A1                   ; NEXT block top address
  CMP.W #10H,A1                 ; block3 top address ?
  JNZ   block_erase_1           ; block erase 0F0000h no complete jump block_erase_1
block_erase_3:                  ; erase block 2(0F8000h)
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_3           ; BUSY jump block_erase_3
  MOV.W #0FH,A1                 ; block-4 top address set
  MOV.W #8000H,A0               ; 008000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0F8000h block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0F8000h block erase verify
block_erase_4:                  ; erase block 2(0FA000h)
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_4           ; BUSY jump block_erase_4

;****************************************************
;+   transmit 1                                     +
;****************************************************
  MOV.W #0,R0                   ; R0 data clear
  MOV.B #0,mode                 ; mode data clear
  MOV.B #0,p_mode               ; p_mode data clear
  MOV.B #0,ring_numb            ; ring_numb data clear
  MOV.B #0,str_numb             ; str_numb data clear
  MOV.B #0,read_numb            ; read_numb data clear
  MOV.W #0FFh,read_cmd          ; read command set
  MOV.W #0,A1                   ; A1 data clear
  MOV.B #28H,UxTB               ; START code send
transmit1_start_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_start_check
  MOV.B #30H,UxTB               ; NORMAL code send
transmit1_normal_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_normal_check
  MOV.B #29H,UxTB               ; END code send
transmit1_end_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_end_check

;****************************************************
;+   program                                        +
;****************************************************
program:
.if DEBUG==1
  MOV.B #0FFh,PD4               ; LED for debugging HND
  MOV.B #000h,P4
.endif
  MOV.B #000h,A0

  CMP.B ring_numb,read_numb     ; download interrupt data number check
  JEQ   program
  MOV.B read_numb,A0            ; read_numb set
  MOV.B dat_buff[A0],R0L       ; dat_buff data 1byte get
  ADD.B #1,str_numb             ; str_numb +1
  ADD.B #1,read_numb            ; read_numb +1
mode_check:
  CMP.B #0,mode                 ; mode check
  JEQ   start_check             ; start code check
  CMP.B #1,mode                 ; mode check
  JEQ   address_set             ; start address check
  CMP.B #2,mode                 ; mode check
  JEQ   length_set              ; length data check
  CMP.B #3,mode                 ; mode check
  JEQ   data_program            ; bin data check
  CMP.B #4,mode                 ; mode check
  JEQ   end_check               ; end code check
  JMP   program

;****************************************************
;+   start check                                    +
;****************************************************
start_check:
  MOV.B str_numb,A0             ; str_numb set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #3,str_numb             ; receive top data 3byte check
  JLTU  start_check_end
  MOV.B #0,info+3
  CMP.B #28h,info               ; 2nd Start code ?
  JNZ   start_check_end
  CMP.B #30h,info+1             ; 2nd DWNH code ?
  JNZ   start_check_end
  CMP.B #42h,info+2             ; 2nd DWNH code ?
  JNZ   start_check_end
  MOV.B #1,mode                 ; address_set mode set
  MOV.W #0,str_numb             ; stock data clear
start_check_end:
  JMP   program

;****************************************************
;+   address set                                    +
;****************************************************
address_set:
  MOV.W #0,prog_addr            ; prog_address clear
  MOV.W #0,prog_addr+2          ; prog_address clear
  MOV.B str_numb,A0             ; str_numb set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #6,str_numb             ; start address data 6byte check
  JLTU  address_set_end
  MOV.W #info,A0                ; high address get
  MOV.B #6,R1L                  ; data set
  JSR   ascBinCnv
  MOV.W #info,A0
  JSR   addrCnv
  MOV.W tmp_addr,prog_addr      ; tmp_adress set
  MOV.W tmp_addr+2,prog_addr+2  ; tmp_address set
  MOV.B #2,mode                 ; length mode set
  MOV.B #0,str_numb             ; stock data clear
address_set_end:
  JMP   program

;****************************************************
;+   length set                                     +
;****************************************************
length_set:
  MOV.W #0,dat_length           ; dat_length buffer clear
  MOV.B str_numb,A0             ; str count set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #4,str_numb             ; length data 4byte check
  JLTU  length_set_end
  MOV.W #info,A0
  MOV.B #4,R1L
  JSR   ascBinCnv
  MOV.W #info-2,A0
  JSR   addrCnv
  MOV.W tmp_addr,dat_length     ; tmp_adress set
  MOV.B #3,mode                 ; data program mode set
  MOV.B #0,str_numb             ; stock data clear
length_set_end:
  JMP   program

;****************************************************
;+   data program                                   +
;****************************************************
data_program:
  CMP.B #0,p_mode
  JNE   data_program_3
  MOV.B #2,p_mode               ; ff_set mode set
  MOV.W prog_addr,page_addr     ; program address set
  MOV.W prog_addr+2,page_addr+2 ; program address set
data_program_3:
  CMP.B #2,p_mode               ; ff_set mode ?
  JNE   data_program_4
  MOV.B #3,p_mode               ; ESC code check mode
  JSR      ff_set
data_program_4:
  CMP.B #1BH,R0L                ; ESC code ?
  JNE   data_program_5
  MOV.B #40H,R0H                ; Bin data ?
  JMP   data_program_end
data_program_5:
  SUB.B R0H,R0L                 ; data get
  MOV.B #0,R0H                  ; data high 8bit clear
  MOV.B page_addr,A0            ; page address set
  MOV.B R0L,data[A0]            ; data set
  SUB.W #1,dat_length           ; data length -1
  CMP.W #0,dat_length           ; data length 0 ?
  JNE   data_program_6
  MOV.B #4,mode                 ; end_check mode set
  MOV.B #0,p_mode               ; 1 page write mode @001
  JSR   page_program
  MOV.B #0,str_numb             ; stock data clear
  JMP   data_program_end
data_program_6:
  CMP.B #0FFH,page_addr         ; page address END ?
  JNE   data_program_7
  JSR   page_program            ; page write go
  MOV.B #2,p_mode               ; buffer FF write mode 
data_program_7:
  ADD.W #1,page_addr            ; next page address set
  ADCF.B page_addr+2            ; next page address set
data_program_end:
  JMP   program

;****************************************************
;+   end check                                      +
;****************************************************
end_check:
  CMP.B #3,str_numb             ; receive end data 3byte check
  JNE   end_check_end
  CMP.B #2EH,R0L                ; RCVE command check
  JEQ   end_sequence
  MOV.B #1,mode                 ; address set mode
  MOV.B #1,str_numb             ; stock data 1
  JMP   address_set
end_check_end:
  JMP   program
end_sequence:

;****************************************************
;+   transmit 2                                     +
;****************************************************
  MOV.B #28H,UxTB               ; START code send
transmit2_start_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_start_check
  MOV.B #30H,UxTB               ; NORMAL code send
transmit2_normal_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_normal_check
  MOV.B #29H,UxTB               ; END code send
transmit2_end_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_end_check
transmit2_send_check:
  BTST  3,UxC0                  ; data send check
  JZ    transmit2_send_check

;****************************************************
;+   software reset                                 +
;****************************************************
  MOV.B #2,PRCR                 ; protect off
  MOV.W #0AAAAH,brg_chk_flg     ; auto baud rate set ok flag
  MOV.B #00001000B,PM0          ; softwear reset
  NOP
  NOP
  NOP
  NOP
  NOP
  NOP

;****************************************************
;+   ff_set soubroutine                             +
;****************************************************
ff_set:
  MOV.W #0,A0                   ; data clear
  MOV.W #80H,R2                 ; write count
ff_set_loop:
  MOV.W #0FFFFH,data[A0]        ; 0FFFFh data write
  ADD.W #2,A0                   ; next 1byte area
  SBJNZ.W  #1,R2,ff_set_loop    ; write count end?
  MOV.W prog_addr,A0               
  BTST  0,A0                    ; program start address is even?
  JZ    ff_set_end                       
  BCLR  0,A0                    ; not even
  MOV.W prog_addr+2,A1          ; 
  JSR   page_read_mode          ; read mode set
  STE.W read_cmd,[A1A0]               ; read command write
  LDE.B [A1A0],R1L              ; program start address -1 read
  AND.W #0FFH,A0                ; 
  MOV.B R1L,data[A0]            ; Eventh data set
  MOV.B #0,prog_addr            ; page address clear
    
ff_set_end:
  RTS

;****************************************************
;+   page program soubroutine                       +
;****************************************************
page_program:
  MOV.W #0,R3                   ; data clear
  MOV.W page_addr,A0            ; page addr middle address
  MOV.W page_addr+2,R3          ; page addr high address
.if DEBUG==1
;  AND.W #0007h,R3
.endif
  MOV.W R3,A1                   ; page addr high address
  BTST  0,A0                    ; program end address is odd?
  JNZ   page_program_mask       ; 
  INC.W A0                      ; no odd
  
  JSR   page_read_mode          ; read mode set
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],R1               ; program end address + 1 read
  AND.W #0FFH,A0                ; 
  MOV.B R1L,data[A0]            ; Oddth data write
  MOV.W page_addr,A0            ; page addr middle address reload
  
  
page_program_mask:
  AND.W #0FF00H,A0              ; page addr middle addres mask
;****************************************************
;+   EXTERNAL memory check soubroutine              +
;****************************************************
page_program_internal:
  CMP.B #0FH,A1                 ; 0C0000-0E0000 area ?
  JLTU  page_program_write      ; small area => page program OK
  CMP.W #0DE00H,A0              ; USER vector move area ?
  JLTU  page_program_write      ; small area => page program OK
  CMP.W #0FE00H,A0              ; monitor vector area ?
  JLTU  page_program_end        ; small area => page program NG
  SUB.W #2000H,A0               ; USER vector area change move

page_program_write:
  MOV.W #80H,R2                 ; 256byte write count
  MOV.W #0,A1
  PUSH.W R0
  MOV.W #40h,R0
  
;----------------------------------------------------
; HND write                                         +
;----------------------------------------------------
page_program_HND:
  MOV.W data[A1],R1             ; Program data read
  CMP.W #0FFFFh,R1
  JZ    next_program
  PUSH.W A1                     ; A0 push
  MOV.W R3,A1                   ; page address high 4bit again
  STE.W R0,[A1A0]               ; Program command write
  STE.W R1,[A1A0]               ; data write
  POP.W A1                      ; A0 pop
page_write_HND_check:           ;
  BTST  0,FMCR                  ; busy ?
  JZ    page_write_HND_check

  BTST  6,FMCR                  ; program error?
  JZ    next_program                     
  PUSH.W R0                     ; R0 push
  MOV.W #50H,R0                 ; clear status command set
  STE.W R0,0F0000H              ; clear status command write
  POP.W R0                      ; R0 pop
page_program_error:
.if DEBUG==1
  MOV.B #0FFh,P4                ; LED for debugging HND
.endif
  JMP   page_program_error      ; error loop (KD30 time out)
next_program:
  ADD.W #2,A0                   ; Program address next 2byte
  ADD.B #2,A1                   ; next data count set
  SUB.W #1,R2                   ;
  JNZ   page_program_HND        ; Page write end?
  POP.W R0
page_program_end:
  RTS

;****************************************************
;+   load receive interrupt                         +
;****************************************************
load_receive:
  FSET  B                       ; BANK 1 register use
  MOV.B ring_numb,A0            ; ring_numb set
  MOV.B UxRB,dat_buff[A0]      ; load data set
  ADD.B #1,ring_numb            ; ring_numb +1
  FCLR  B                       ; BANK 0 change
  REIT

;----------------------------------------------------
;+  Address convert                                 +
;+     arg : a0  start address                      +
;+    rtn : tmp_addr                                +
;----------------------------------------------------
addrCnv:
  MOVLL 5[A0],R0L               ; low address low 4bit
  MOVLH 4[A0],R0L               ; low address high 4bit
  MOV.B R0L,tmp_addr            ; low address set
  MOVLL 3[A0],R0L               ; middle address low 4bit
  MOVLH 2[A0],R0L               ; middle address high 4bit
  MOV.B R0L,tmp_addr+1          ; middle set
  MOVLL 1[A0],R0L               ; high address low 4bit
  MOVLH [A0],R0L                ; high address high 4bit
  MOV.B R0L,tmp_addr+2          ; high address set
  MOV.B #0,tmp_addr+3           ; high high address ZERO set
  RTS

;----------------------------------------------------
;+  Address convert 5bytes                          +
;+     arg : a0  start address                      +
;+    rtn : tmp_addr                                +
;----------------------------------------------------
addrCnv5:
  MOVLL 4[A0],R0L               ; low address low 4bit
  MOVLH 3[A0],R0L               ; low address high 4bit
  MOV.B R0L,tmp_addr            ; low address set
  MOVLL 2[A0],R0L               ; middle address low 4bit
  MOVLH 1[A0],R0L               ; middle address high 4bit
  MOV.B R0L,tmp_addr+1          ; middle set
  XOR.B R0L,R0L
  MOVLL [A0],R0L                ; high address
  MOV.B R0L,tmp_addr+2          ; high address set
  MOV.B #0,tmp_addr+3
  RTS

;----------------------------------------------------
;+  Asc --> Bin convert                             +
;+    arg1 : a0  start address                      +
;+    arg2 : r1l convert times                      +
;+    rtn  : -                                      +
;----------------------------------------------------
ascBinCnv:
  MOV.B [A0],R0L
  CMP.B #40H,R0L
  JC    ascBinCnv_chk
  SUB.B #30H,R0L
  JMP   ascBin_next
ascBinCnv_chk:
  SUB.B #37H,R0L
ascBin_next:
  MOV.B R0L,[A0]
  ADD.W #1,A0
  SBJNZ.B #1,R1L,ascBinCnv
  RTS
    
;****************************************************
;+   read mode set                                  +
;****************************************************
page_read_mode:
  PUSH.W A0                     ; push a0
  PUSH.W A1                     ; push a1
  MOV.W #READ_TEST_DATA & 0FFFFH,A0   ; READ_TEST_DATA address set
  MOV.W #READ_TEST_DATA >> 16,A1 
page_read_1:
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],read_data
  CMP.W #0FFFFH,read_data
  JNZ   page_read_1
  ADD.W #2,A0
page_read_2:
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],read_data
  CMP.W #0FFFFH,read_data
  JNZ   page_read_2
  POP.W A1
  POP.W A0
  RTS
  
;----------------------------------------------------
;+   download module for flash                      +
;----------------------------------------------------
  NOP
  NOP
  NOP
load_prog_end:
;****************************************************
;+   end download module for flash HND type 1       +
;****************************************************
  .ENDM

PROG_HND2  .MACRO
;****************************************************
;+   download module for flash HND type 2           +
;****************************************************
load_prog:
;****************************************************
;+  initialize                                      +
;****************************************************
  LDC   #ISP_ADDR,ISP           ; isp reset
  MOV.W #(load_receive-load_prog+ram_prog),A0 ; UART1 interrupt address
.if UART==0
  MOV.W A0,F_INTB+18*4          ; UART0 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+18*4+2      ; UART0 interrupt vector address(H)
.endif
.if UART==1
  MOV.W A0,F_INTB+20*4          ; UART1 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+20*4+2      ; UART1 interrupt vector address(H)
.endif
.if UART==2
  MOV.W A0,F_INTB+16*4          ; UART1 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+16*4+2      ; UART1 interrupt vector address(H)
.endif
  LDINTB #F_INTB
  MOV.B #00000011B,PRCR         ; PRC0/PRC1 protect off
  MOV.B #00000000B,CM0          ; main clock CM16,CM17 set
.if PLL_MUL > 1
  MOV.W brg_data,R0             ; read value of multiplier
  INC.B R0L
  DIV.B #PLL_MUL                ; calculate new value for BRG
  DEC.B R0L
  MOV.B R0L,UxBRG               ; write new value for BRG
  MOV.B #00100000B,CM1          ; clock = f(Xin)/1
.else
.if TS==1
  MOV.B #01101000B,CM1          ; 1/2 cycle
.else
  MOV.B #01100000B,CM1          ; 1/2 cycle
.endif
.endif
  MOV.B #10001000B,PM1          ; 1 wait & RAM 23K EXT
  MOV.B #00000000B,PRCR         ; protect on
  MOV.B #00000000B,FMCR         ; flash memory control register
  MOV.B #00000010B,FMCR         ; flash memory control register(CPU Write mode)
  MOV.B #00000110B,FMCR         ; flash memory control register(rock bit off)
  FSET  I

;****************************************************
;+  block erase                                     +
;****************************************************
block_erase:
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase             ; C = 0 jump block_erase
  BCLR  1,FMCR                  ; normal mode
  BSET  1,FMCR                  ; CPU write mode
  MOV.W #FLASH_START,A1         ; block6-3 top address set
block_erase_1:                  ; erase block 6-5(0C0000h-0F0000h)
  MOV.W #0000H,A0               ; 0000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0xFFFEh block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0xFFFEh block erase verify
block_erase_2:
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_2           ; BUSY jump block_erase_2
  ADD.W #1,A1                   ; NEXT block top address
  CMP.W #10H,A1                 ; block3 top address ?
  JNZ   block_erase_1           ; block erase 0F0000h no complete jump block_erase_1
block_erase_3:                  ; erase block 2(0F8000h)
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_3           ; BUSY jump block_erase_3
  MOV.W #0FH,A1                 ; block-4 top address set
  MOV.W #8000H,A0               ; 008000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0F8000h block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0F8000h block erase verify
block_erase_4:                  ; erase block 2(0FA000h)
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_4           ; BUSY jump block_erase_4
  MOV.W #0FH,A1                 ; block-3 top address set
  MOV.W #0A000H,A0              ; 00A000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0FA000h block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0FA000h block erase verify
block_erase_5:
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_5
  MOV.W #0FH,A1                 ; block-2 top address set
  MOV.W #0C000H,A0              ; 00C000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0FA000h block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0FC000h block erase verify
block_erase_6:
  BTST  0,FMCR                  ; RDY bit check
  JNC   block_erase_6

;****************************************************
;+   transmit 1                                     +
;****************************************************
  MOV.W #0,R0                   ; R0 data clear
  MOV.B #0,mode                 ; mode data clear
  MOV.B #0,p_mode               ; p_mode data clear
  MOV.B #0,ring_numb            ; ring_numb data clear
  MOV.B #0,str_numb             ; str_numb data clear
  MOV.B #0,read_numb            ; read_numb data clear
  MOV.W #0FFh,read_cmd          ; read command set
  MOV.W #0,A1                   ; A1 data clear
  MOV.B #28H,UxTB               ; START code send
transmit1_start_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_start_check
  MOV.B #30H,UxTB               ; NORMAL code send
transmit1_normal_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_normal_check
  MOV.B #29H,UxTB               ; END code send
transmit1_end_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_end_check

;****************************************************
;+   program                                        +
;****************************************************
program:
  CMP.B ring_numb,read_numb     ; download interrupt data number check
  JEQ   program
  MOV.B read_numb,A0            ; read_numb set
  MOV.B dat_buff[A0],R0L       ; dat_buff data 1byte get
  ADD.B #1,str_numb             ; str_numb +1
  ADD.B #1,read_numb            ; read_numb +1
mode_check:
  CMP.B #0,mode                 ; mode check
  JEQ   start_check             ; start code check
  CMP.B #1,mode                 ; mode check
  JEQ   address_set             ; start address check
  CMP.B #2,mode                 ; mode check
  JEQ   length_set              ; length data check
  CMP.B #3,mode                 ; mode check
  JEQ   data_program            ; bin data check
  CMP.B #4,mode                 ; mode check
  JEQ   end_check               ; end code check
  JMP   program

;****************************************************
;+   start check                                    +
;****************************************************
start_check:
  MOV.B str_numb,A0             ; str_numb set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #3,str_numb             ; receive top data 3byte check
  JLTU  start_check_end
  MOV.B #0,info+3
  CMP.B #28h,info               ; 2nd Start code ?
  JNZ   start_check_end
  CMP.B #30h,info+1             ; 2nd DWNH code ?
  JNZ   start_check_end
  CMP.B #42h,info+2             ; 2nd DWNH code ?
  JNZ   start_check_end
  MOV.B #1,mode                 ; address_set mode set
  MOV.W #0,str_numb             ; stock data clear
start_check_end:
  JMP   program

;****************************************************
;+   address set                                    +
;****************************************************
address_set:
  MOV.W #0,prog_addr            ; prog_address clear
  MOV.W #0,prog_addr+2          ; prog_address clear
  MOV.B str_numb,A0             ; str_numb set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #6,str_numb             ; start address data 6byte check
  JLTU  address_set_end
  MOV.W #info,A0                ; high address get
  MOV.B #6,R1L                  ; data set
  JSR   ascBinCnv
  MOV.W #info,A0
  JSR   addrCnv
  MOV.W tmp_addr,prog_addr      ; tmp_adress set
  MOV.W tmp_addr+2,prog_addr+2  ; tmp_address set
  MOV.B #2,mode                 ; length mode set
  MOV.B #0,str_numb             ; stock data clear
address_set_end:
  JMP   program

;****************************************************
;+   length set                                     +
;****************************************************
length_set:
  MOV.W #0,dat_length           ; dat_length buffer clear
  MOV.B str_numb,A0             ; str count set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #4,str_numb             ; length data 4byte check
  JLTU  length_set_end
  MOV.W #info,A0
  MOV.B #4,R1L
  JSR   ascBinCnv
  MOV.W #info-2,A0
  JSR   addrCnv
  MOV.W tmp_addr,dat_length     ; tmp_adress set
  MOV.B #3,mode                 ; data program mode set
  MOV.B #0,str_numb             ; stock data clear
length_set_end:
  JMP   program

;****************************************************
;+   data program                                   +
;****************************************************
data_program:
  CMP.B #0,p_mode
  JNE   data_program_3
  MOV.B #2,p_mode               ; ff_set mode set
  MOV.W prog_addr,page_addr     ; program address set
  MOV.W prog_addr+2,page_addr+2 ; program address set
data_program_3:
  CMP.B #2,p_mode               ; ff_set mode ?
  JNE   data_program_4
  MOV.B #3,p_mode               ; ESC code check mode
  JSR      ff_set
data_program_4:
  CMP.B #1BH,R0L                ; ESC code ?
  JNE   data_program_5
  MOV.B #40H,R0H                ; Bin data ?
  JMP   data_program_end
data_program_5:
  SUB.B R0H,R0L                 ; data get
  MOV.B #0,R0H                  ; data high 8bit clear
  MOV.B page_addr,A0            ; page address set
  MOV.B R0L,data[A0]            ; data set
  SUB.W #1,dat_length           ; data length -1
  CMP.W #0,dat_length           ; data length 0 ?
  JNE   data_program_6
  MOV.B #4,mode                 ; end_check mode set
  MOV.B #0,p_mode               ; 1 page write mode @001
  JSR   page_program
  MOV.B #0,str_numb             ; stock data clear
  JMP   data_program_end
data_program_6:
  CMP.B #0FFH,page_addr         ; page address END ?
  JNE   data_program_7
  JSR   page_program            ; page write go
  MOV.B #2,p_mode               ; buffer FF write mode 
data_program_7:
  ADD.W #1,page_addr            ; next page address set
  ADCF.B page_addr+2            ; next page address set
data_program_end:
  JMP   program

;****************************************************
;+   end check                                      +
;****************************************************
end_check:
  CMP.B #3,str_numb             ; receive end data 3byte check
  JNE   end_check_end
  CMP.B #2EH,R0L                ; RCVE command check
  JEQ   end_sequence
  MOV.B #1,mode                 ; address set mode
  MOV.B #1,str_numb             ; stock data 1
  JMP   address_set
end_check_end:
  JMP   program
end_sequence:

;****************************************************
;+   transmit 2                                     +
;****************************************************
  MOV.B #28H,UxTB               ; START code send
transmit2_start_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_start_check
  MOV.B #30H,UxTB               ; NORMAL code send
transmit2_normal_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_normal_check
  MOV.B #29H,UxTB               ; END code send
transmit2_end_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_end_check
transmit2_send_check:
  BTST  3,UxC0                  ; data send check
  JZ    transmit2_send_check

;****************************************************
;+   software reset                                 +
;****************************************************
  MOV.B #00000011B,PRCR         ; protect off
  MOV.W #0AAAAH,brg_chk_flg     ; auto baud rate set ok flag
  MOV.B #00001000B,PM0          ; softwear reset
  NOP
  NOP
  NOP
  NOP
  NOP
  NOP

;****************************************************
;+   ff_set soubroutine                             +
;****************************************************
ff_set:
  MOV.W #0,A0                   ; data clear
  MOV.W #80H,R2                 ; write count
ff_set_loop:
  MOV.W #0FFFFH,data[A0]        ; 0FFFFh data write
  ADD.W #2,A0                   ; next 1byte area
  SBJNZ.W  #1,R2,ff_set_loop    ; write count end?
  MOV.W prog_addr,A0               
  BTST  0,A0                    ; program start address is even?
  JZ    ff_set_end                       
  BCLR  0,A0                    ; not even
  MOV.W prog_addr+2,A1          ; 
  JSR   page_read_mode          ; read mode set
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.B [A1A0],R1L              ; program start address -1 read
  AND.W #0FFH,A0                ; 
  MOV.B R1L,data[A0]            ; Eventh data set
  MOV.B #0,prog_addr            ; page address clear
ff_set_end:
  RTS

;****************************************************
;+   page program soubroutine                       +
;****************************************************
page_program:
  MOV.W #0,R3                   ; data clear
  MOV.W page_addr,A0            ; page addr middle address
  MOV.W page_addr+2,R3          ; page addr high address
  MOV.W R3,A1                   ; page addr high address
  BTST  0,A0                    ; program end address is odd?
  JNZ   page_program_mask       ; 
  INC.W A0                      ; no odd
  
  JSR   page_read_mode          ; read mode set
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],R1               ; program end address + 1 read
  AND.W #0FFH,A0                ; 
  MOV.B R1L,data[A0]            ; Oddth data write
  MOV.W page_addr,A0            ; page addr middle address reload
  
  
page_program_mask:
  AND.W #0FF00H,A0              ; page addr middle addres mask
;****************************************************
;+   EXTERNAL memory check soubroutine              +
;****************************************************
page_program_internal:
  CMP.B #0FH,A1                 ; 0C0000-0E0000 area ?
  JLTU  page_program_write      ; small area => page program OK
  CMP.W #0DE00H,A0              ; USER vector move area ?
  JLTU  page_program_write      ; small area => page program OK
  CMP.W #0FE00H,A0              ; monitor vector area ?
  JLTU  page_program_end        ; small area => page program NG
  SUB.W #2000H,A0               ; USER vector area change move

page_program_write:
  MOV.W #80H,R2                 ; 256byte write count
  MOV.W #0,A1
  PUSH.W R0
  MOV.W #40h,R0
  
;----------------------------------------------------
; HND write                                         +
;----------------------------------------------------
page_program_HND:
  MOV.W data[A1],R1             ; Program data read
  CMP.W #0FFFFh,R1
  JZ    next_program
  PUSH.W A1                     ; A0 push
  MOV.W R3,A1                   ; page address high 4bit again
  STE.W R0,[A1A0]               ; Program command write
  STE.W R1,[A1A0]               ; data write
  POP.W A1                      ; A0 pop
page_write_HND_check:           ;
  BTST  0,FMCR                  ; busy ?
  JZ    page_write_HND_check

  BTST  6,FMCR                  ; program error?
  JZ    next_program                     
  PUSH.W R0                     ; R0 push
  MOV.W #50H,R0                 ; clear statas command set
  STE.W R0,0F0000H              ; clear statas command write
  POP.W R0                      ; R0 pop
page_program_error:
  JMP   page_program_error      ; error loop (KD30 time out)
next_program:
  ADD.W #2,A0                   ; Program address next 2byte
  ADD.B #2,A1                   ; next data count set
  SUB.W #1,R2                   ;
  JNZ   page_program_HND        ; Page write end?
  POP.W R0
page_program_end:
  RTS

;****************************************************
;+   load receive interrupt                         +
;****************************************************
load_receive:
  FSET  B                       ; BANK 1 register use
  MOV.B ring_numb,A0            ; ring_numb set
  MOV.B UxRB,dat_buff[A0]      ; load data set
  ADD.B #1,ring_numb            ; ring_numb +1
  FCLR  B                       ; BANK 0 change
  REIT

;----------------------------------------------------
;+  Address convert                                 +
;+     arg : a0  start address                      +
;+    rtn : tmp_addr                                +
;----------------------------------------------------
addrCnv:
  MOVLL 5[A0],R0L               ; low address low 4bit
  MOVLH 4[A0],R0L               ; low address high 4bit
  MOV.B R0L,tmp_addr            ; low address set
  MOVLL 3[A0],R0L               ; middle address low 4bit
  MOVLH 2[A0],R0L               ; middle address high 4bit
  MOV.B R0L,tmp_addr+1          ; middle set
  MOVLL 1[A0],R0L               ; high address low 4bit
  MOVLH [A0],R0L                ; high address high 4bit
  MOV.B R0L,tmp_addr+2          ; high address set
  MOV.B #0,tmp_addr+3           ; high high address ZERO set
  RTS

;----------------------------------------------------
;+  Address convert 5bytes                          +
;+     arg : a0  start address                      +
;+    rtn : tmp_addr                                +
;----------------------------------------------------
addrCnv5:
  MOVLL 4[A0],R0L               ; low address low 4bit
  MOVLH 3[A0],R0L               ; low address high 4bit
  MOV.B R0L,tmp_addr            ; low address set
  MOVLL 2[A0],R0L               ; middle address low 4bit
  MOVLH 1[A0],R0L               ; middle address high 4bit
  MOV.B R0L,tmp_addr+1          ; middle set
  XOR.B R0L,R0L
  MOVLL [A0],R0L                ; high address
  MOV.B R0L,tmp_addr+2          ; high address set
  MOV.B #0,tmp_addr+3
  RTS

;----------------------------------------------------
;+  Asc --> Bin convert                             +
;+    arg1 : a0  start address                      +
;+    arg2 : r1l convert times                      +
;+    rtn  : -                                      +
;----------------------------------------------------
ascBinCnv:
  MOV.B [A0],R0L
  CMP.B #40H,R0L
  JC    ascBinCnv_chk
  SUB.B #30H,R0L
  JMP   ascBin_next
ascBinCnv_chk:
  SUB.B #37H,R0L
ascBin_next:
  MOV.B R0L,[A0]
  ADD.W #1,A0
  SBJNZ.B #1,R1L,ascBinCnv
  RTS
    
;****************************************************
;+   read mode set                                  +
;****************************************************
page_read_mode:
  PUSH.W A0                     ; push a0
  PUSH.W A1                     ; push a1
  MOV.W #READ_TEST_DATA & 0FFFFH,A0   ; READ_TEST_DATA address set
  MOV.W #READ_TEST_DATA >> 16,A1 
page_read_1:
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],read_data
  CMP.W #0FFFFH,read_data
  JNZ   page_read_1
  ADD.W #2,A0
page_read_2:
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],read_data
  CMP.W #0FFFFH,read_data
  JNZ   page_read_2
  
  POP.W A1
  POP.W A0
  RTS
  
;----------------------------------------------------
;+   download module for flash                      +
;----------------------------------------------------
  NOP
  NOP
  NOP
load_prog_end:
;****************************************************
;+   end download module for flash HND type 2       +
;****************************************************
  .ENDM

PROG_HND3  .MACRO
;****************************************************
;+   download module for flash HNDc type            +
;****************************************************
load_prog:
;****************************************************
;+  initialize                                      +
;****************************************************
  LDC   #ISP_ADDR,ISP           ; isp reset
  MOV.W #(load_receive-load_prog+ram_prog),A0 ; UART1 interrupt address
.if UART==0
  MOV.W A0,F_INTB+18*4          ; UART0 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+18*4+2      ; UART0 interrupt vector address(H)
.endif
.if UART==1
  MOV.W A0,F_INTB+20*4          ; UART1 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+20*4+2      ; UART1 interrupt vector address(H)
.endif
.if UART==2
  MOV.W A0,F_INTB+16*4          ; UART2 interrupt vector address(M,L)
  MOV.W #00H,F_INTB+16*4+2      ; UART2 interrupt vector address(H)
.endif
  LDINTB #F_INTB
  MOV.B #00000011B,PRCR         ; PRC0/PRC1 protect off
  MOV.B #00000000B,CM0          ; main clock CM16,CM17 set
  MOV.B #01100000B,CM1          ; 1/2 cycle
  MOV.B #10001000B,PM1          ; 1 wait & RAM 23K EXT
  MOV.B #00000000B,PRCR         ; protect on

  BCLR  3,FMR0                  ; reset flash memory and flash control
  BSET  3,FMR0                  ; reset flash memory and flash control
?:
  BTST  0,FMR0
  JZ    ?-
  BCLR  3,FMR0                  ; start flash memory again
?:
  BTST  0,FMR0
  JZ    ?-
?:
  BCLR  1,FMR0                  ; flash memory control register
  BSET  1,FMR0                  ; flash memory control register(CPU rewrite mode)
  NOP
  NOP
  NOP
  NOP
  BTST  1,FMR0
  JZ    ?-
  BCLR  2,FMR0                  ; flash memory control register
  BSET  2,FMR0                  ; flash memory control register(enable 8K blocks)
  FSET  I
;****************************************************
;+  block erase                                     +
;****************************************************
block_erase:
  BTST  0,FMR0                  ; RDY bit check
  JNC   block_erase             ; C = 0 jump block_erase
  MOV.W #FLASH_START,A1         ; block3 top address set
block_erase_1:                  ; erase block 3(0F0000h-0F7FFFh)
  MOV.W #0000H,A0               ; 0000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0xFFFEh block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0xFFFEh block erase verify
block_erase_2:
  BTST  0,FMR0                  ; RDY bit check
  JNC   block_erase_2           ; BUSY jump block_erase_2
  ADD.W #1,A1                   ; NEXT block top address
  CMP.W #10H,A1                 ; block3 top address ?
  JNZ   block_erase_1           ; block erase 0F0000h no complete jump block_erase_1
block_erase_3:                  ; erase block 2(0F8000h)
  BTST  0,FMR0                  ; RDY bit check
  JNC   block_erase_3           ; BUSY jump block_erase_3
  MOV.W #0FH,A1                 ; block 2 top address set
  MOV.W #8000H,A0               ; 008000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0F8000h block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0F8000h block erase verify
block_erase_4:                  ; erase block 1(0FC000h)
  BTST  0,FMR0                  ; RDY bit check
  JNC   block_erase_4           ; BUSY jump block_erase_4
  MOV.W #0FH,A1                 ; block 1 top address set    ######### does not work !!!!!!!!!
  MOV.W #0C000H,A0              ; 00C000h set
  MOV.W #20H,R0                 ; block erase command code set
  STE.W R0,[A1A0]               ; 0FA000h block erase start
  MOV.W #0D0H,R1                ; verify command code set
  STE.W R1,[A1A0]               ; 0FA000h block erase verify
block_erase_5:
  BTST  0,FMR0                  ; RDY bit check
  JNC   block_erase_5

  BCLR  3,FMR0                  ; reset flash memory and flash control
  BSET  3,FMR0                  ; reset flash memory and flash control
?:
  BTST  0,FMR0
  JZ    ?-

  BCLR  3,FMR0                  ; start flash memory again
?:
  BTST  0,FMR0
  JZ    ?-

  BCLR  1,FMR0                  ; flash memory control register
  BSET  1,FMR0                  ; flash memory control register(CPU rewrite mode)
  BTST  1,FMR0
  JZ    ?-
  BCLR  2,FMR0                  ; flash memory control register
  BSET  2,FMR0                  ; flash memory control register(enable 8K blocks)
  
;****************************************************
;+   transmit 1                                     +
;****************************************************
  MOV.W #0,R0                   ; R0 data clear
  MOV.B #0,mode                 ; mode data clear
  MOV.B #0,p_mode               ; p_mode data clear
  MOV.B #0,ring_numb            ; ring_numb data clear
  MOV.B #0,str_numb             ; str_numb data clear
  MOV.B #0,read_numb            ; read_numb data clear
  MOV.W #0FFh,read_cmd          ; read command set
  MOV.W #0,A1                   ; A1 data clear
  MOV.B #28H,UxTB               ; START code send
transmit1_start_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_start_check
  MOV.B #30H,UxTB               ; NORMAL code send
transmit1_normal_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_normal_check
  MOV.B #29H,UxTB               ; END code send
transmit1_end_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit1_end_check

;****************************************************
;+   program                                        +
;****************************************************
program:
  CMP.B ring_numb,read_numb     ; download interrupt data number check
  JEQ   program
  MOV.B read_numb,A0            ; read_numb set
  MOV.B dat_buff[A0],R0L       ; dat_buff data 1byte get
  ADD.B #1,str_numb             ; str_numb +1
  ADD.B #1,read_numb            ; read_numb +1
mode_check:
  CMP.B #0,mode                 ; mode check
  JEQ   start_check             ; start code check
  CMP.B #1,mode                 ; mode check
  JEQ   address_set             ; start address check
  CMP.B #2,mode                 ; mode check
  JEQ   length_set              ; length data check
  CMP.B #3,mode                 ; mode check
  JEQ   data_program            ; bin data check
  CMP.B #4,mode                 ; mode check
  JEQ   end_check               ; end code check
  JMP   program

;****************************************************
;+   start check                                    +
;****************************************************
start_check:
  MOV.B str_numb,A0             ; str_numb set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #3,str_numb             ; receive top data 3byte check
  JLTU  start_check_end
  MOV.B #0,info+3
  CMP.B #28h,info               ; 2nd Start code ?
  JNZ   start_check_end
  CMP.B #30h,info+1             ; 2nd DWNH code ?
  JNZ   start_check_end
  CMP.B #42h,info+2             ; 2nd DWNH code ?
  JNZ   start_check_end
  MOV.B #1,mode                 ; address_set mode set
  MOV.W #0,str_numb             ; stock data clear
start_check_end:
  JMP   program

;****************************************************
;+   address set                                    +
;****************************************************
address_set:
  MOV.W #0,prog_addr            ; prog_address clear
  MOV.W #0,prog_addr+2          ; prog_address clear
  MOV.B str_numb,A0             ; str_numb set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #6,str_numb             ; start address data 6byte check
  JLTU  address_set_end
  MOV.W #info,A0                ; high address get
  MOV.B #6,R1L                  ; data set
  JSR   ascBinCnv
  MOV.W #info,A0
  JSR   addrCnv
  MOV.W tmp_addr,prog_addr      ; tmp_adress set
  MOV.W tmp_addr+2,prog_addr+2  ; tmp_address set
  MOV.B #2,mode                 ; length mode set
  MOV.B #0,str_numb             ; stock data clear
address_set_end:
  JMP   program

;****************************************************
;+   length set                                     +
;****************************************************
length_set:
  MOV.W #0,dat_length           ; dat_length buffer clear
  MOV.B str_numb,A0             ; str count set
  MOV.B R0L,info-1[A0]          ; down load data store
  CMP.B #4,str_numb             ; length data 4byte check
  JLTU  length_set_end
  MOV.W #info,A0
  MOV.B #4,R1L
  JSR   ascBinCnv
  MOV.W #info-2,A0
  JSR   addrCnv
  MOV.W tmp_addr,dat_length     ; tmp_adress set
  MOV.B #3,mode                 ; data program mode set
  MOV.B #0,str_numb             ; stock data clear
length_set_end:
  JMP   program

;****************************************************
;+   data program                                   +
;****************************************************
data_program:
  CMP.B #0,p_mode
  JNE   data_program_3
  MOV.B #2,p_mode               ; ff_set mode set
  MOV.W prog_addr,page_addr     ; program address set
  MOV.W prog_addr+2,page_addr+2 ; program address set
data_program_3:
  CMP.B #2,p_mode               ; ff_set mode ?
  JNE   data_program_4
  MOV.B #3,p_mode               ; ESC code check mode
  JSR      ff_set
data_program_4:
  CMP.B #1BH,R0L                ; ESC code ?
  JNE   data_program_5
  MOV.B #40H,R0H                ; Bin data ?
  JMP   data_program_end
data_program_5:
  SUB.B R0H,R0L                 ; data get
  MOV.B #0,R0H                  ; data high 8bit clear
  MOV.B page_addr,A0            ; page address set
  MOV.B R0L,data[A0]            ; data set
  SUB.W #1,dat_length           ; data length -1
  CMP.W #0,dat_length           ; data length 0 ?
  JNE   data_program_6
  MOV.B #4,mode                 ; end_check mode set
  MOV.B #0,p_mode               ; 1 page write mode @001
  JSR   page_program
  MOV.B #0,str_numb             ; stock data clear
  JMP   data_program_end
data_program_6:
  CMP.B #0FFH,page_addr         ; page address END ?
  JNE   data_program_7
  JSR   page_program            ; page write go
  MOV.B #2,p_mode               ; buffer FF write mode 
data_program_7:
  ADD.W #1,page_addr            ; next page address set
  ADCF.B page_addr+2            ; next page address set
data_program_end:
  JMP   program

;****************************************************
;+   end check                                      +
;****************************************************
end_check:
  CMP.B #3,str_numb             ; receive end data 3byte check
  JNE   end_check_end
  CMP.B #2EH,R0L                ; RCVE command check
  JEQ   end_sequence
  MOV.B #1,mode                 ; address set mode
  MOV.B #1,str_numb             ; stock data 1
  JMP   address_set
end_check_end:
  JMP   program
end_sequence:

;****************************************************
;+   transmit 2                                     +
;****************************************************
  MOV.B #28H,UxTB               ; START code send
transmit2_start_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_start_check
  MOV.B #30H,UxTB               ; NORMAL code send
transmit2_normal_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_normal_check
  MOV.B #29H,UxTB               ; END code send
transmit2_end_check:
  BTST  1,UxC1                  ; data send check
  JZ    transmit2_end_check
transmit2_send_check:
  BTST  3,UxC0                  ; data send check
  JZ    transmit2_send_check

;****************************************************
;+   software reset                                 +
;****************************************************
  MOV.B #00000011B,PRCR                 ; protect off
  MOV.W #0AAAAH,brg_chk_flg     ; auto baud rate set ok flag
  MOV.B #00001000B,PM0          ; softwear reset
  NOP
  NOP
  NOP
  NOP
  NOP
  NOP

;****************************************************
;+   ff_set soubroutine                             +
;****************************************************
ff_set:
  MOV.W #0,A0                   ; data clear
  MOV.W #80H,R2                 ; write count
ff_set_loop:
  MOV.W #0FFFFH,data[A0]        ; 0FFFFh data write
  ADD.W #2,A0                   ; next 1byte area
  SBJNZ.W  #1,R2,ff_set_loop    ; write count end?
  MOV.W prog_addr,A0               
  BTST  0,A0                    ; program start address is even?
  JZ    ff_set_end                       
  BCLR  0,A0                    ; not even
  MOV.W prog_addr+2,A1          ; 
  JSR   page_read_mode          ; read mode set
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.B [A1A0],R1L              ; program start address -1 read
  AND.W #0FFH,A0                ; 
  MOV.B R1L,data[A0]            ; Eventh data set
  MOV.B #0,prog_addr            ; page address clear
ff_set_end:
  RTS

;****************************************************
;+   page program soubroutine                       +
;****************************************************
page_program:
  MOV.W #0,R3                   ; data clear
  MOV.W page_addr,A0            ; page addr middle address
  MOV.W page_addr+2,R3          ; page addr high address
  MOV.W R3,A1                   ; page addr high address
  BTST  0,A0                    ; program end address is odd?
  JNZ   page_program_mask       ; 
  INC.W A0                      ; no odd
  
  JSR   page_read_mode          ; read mode set
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],R1               ; program end address + 1 read
  AND.W #0FFH,A0                ; 
  MOV.B R1L,data[A0]            ; Oddth data write
  MOV.W page_addr,A0            ; page addr middle address reload
  
  
page_program_mask:
  AND.W #0FF00H,A0              ; page addr middle addres mask
;****************************************************
;+   EXTERNAL memory check soubroutine              +
;****************************************************
page_program_internal:
  CMP.B #0FH,A1                 ; 0C0000-0E0000 area ?
  JLTU  page_program_write      ; small area => page program OK
  CMP.W #0DE00H,A0              ; USER vector move area ?
  JLTU  page_program_write      ; small area => page program OK
  CMP.W #0FE00H,A0              ; monitor vector area ?
  JLTU  page_program_end        ; small area => page program NG
  SUB.W #2000H,A0               ; USER vector area change move

page_program_write:
  MOV.W #80H,R2                 ; 256byte write count
  MOV.W #0,A1
  PUSH.W R0
  MOV.W #40h,R0
  
;----------------------------------------------------
; HND write                                         +
;----------------------------------------------------
page_program_HND:
  MOV.W data[A1],R1             ; Program data read
  CMP.W #0FFFFh,R1
  JZ    next_program
  PUSH.W A1                     ; A0 push
  MOV.W R3,A1                   ; page address high 4bit again
  STE.W R0,[A1A0]               ; Program command write
  STE.W R1,[A1A0]               ; data write
  POP.W A1                      ; A0 pop
page_write_HND_check:           ;
  BTST  0,FMR0                  ; busy ?
  JZ    page_write_HND_check

  BTST  6,FMR0                  ; program error?
  JZ    next_program                     
  PUSH.W R0                     ; R0 push
  MOV.W #50H,R0                 ; clear statas command set
  STE.W R0,0F0000H              ; clear statas command write
  POP.W R0                      ; R0 pop
page_program_error:
  JMP   page_program_error      ; error loop (KD30 time out)
next_program:
  ADD.W #2,A0                   ; Program address next 2byte
  ADD.B #2,A1                   ; next data count set
  SUB.W #1,R2                   ;
  JNZ   page_program_HND        ; Page write end?
  POP.W R0
page_program_end:
  RTS

;****************************************************
;+   load receive interrupt                         +
;****************************************************
load_receive:
  FSET  B                       ; BANK 1 register use
  MOV.B ring_numb,A0            ; ring_numb set
  MOV.B UxRB,dat_buff[A0]      ; load data set
  ADD.B #1,ring_numb            ; ring_numb +1
  FCLR  B                       ; BANK 0 change
  REIT

;----------------------------------------------------
;+  Address convert                                 +
;+     arg : a0  start address                      +
;+    rtn : tmp_addr                                +
;----------------------------------------------------
addrCnv:
  MOVLL 5[A0],R0L               ; low address low 4bit
  MOVLH 4[A0],R0L               ; low address high 4bit
  MOV.B R0L,tmp_addr            ; low address set
  MOVLL 3[A0],R0L               ; middle address low 4bit
  MOVLH 2[A0],R0L               ; middle address high 4bit
  MOV.B R0L,tmp_addr+1          ; middle set
  MOVLL 1[A0],R0L               ; high address low 4bit
  MOVLH [A0],R0L                ; high address high 4bit
  MOV.B R0L,tmp_addr+2          ; high address set
  MOV.B #0,tmp_addr+3           ; high high address ZERO set
  RTS

;----------------------------------------------------
;+  Address convert 5bytes                          +
;+     arg : a0  start address                      +
;+    rtn : tmp_addr                                +
;----------------------------------------------------
addrCnv5:
  MOVLL 4[A0],R0L               ; low address low 4bit
  MOVLH 3[A0],R0L               ; low address high 4bit
  MOV.B R0L,tmp_addr            ; low address set
  MOVLL 2[A0],R0L               ; middle address low 4bit
  MOVLH 1[A0],R0L               ; middle address high 4bit
  MOV.B R0L,tmp_addr+1          ; middle set
  XOR.B R0L,R0L
  MOVLL [A0],R0L                ; high address
  MOV.B R0L,tmp_addr+2          ; high address set
  MOV.B #0,tmp_addr+3
  RTS

;----------------------------------------------------
;+  Asc --> Bin convert                             +
;+    arg1 : a0  start address                      +
;+    arg2 : r1l convert times                      +
;+    rtn  : -                                      +
;----------------------------------------------------
ascBinCnv:
  MOV.B [A0],R0L
  CMP.B #40H,R0L
  JC    ascBinCnv_chk
  SUB.B #30H,R0L
  JMP   ascBin_next
ascBinCnv_chk:
  SUB.B #37H,R0L
ascBin_next:
  MOV.B R0L,[A0]
  ADD.W #1,A0
  SBJNZ.B #1,R1L,ascBinCnv
  RTS
    
;****************************************************
;+   read mode set                                  +
;****************************************************
page_read_mode:
  PUSH.W A0                     ; push a0
  PUSH.W A1                     ; push a1
  MOV.W #READ_TEST_DATA & 0FFFFH,A0   ; READ_TEST_DATA address set
  MOV.W #READ_TEST_DATA >> 16,A1 
page_read_1:
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],read_data
  CMP.W #0FFFFH,read_data
  JNZ   page_read_1
  ADD.W #2,A0
page_read_2:
  STE.W read_cmd,[A1A0]         ; read command write
  LDE.W [A1A0],read_data
  CMP.W #0FFFFH,read_data
  JNZ   page_read_2
  
  POP.W A1
  POP.W A0
  RTS
  
;----------------------------------------------------
;+   download module for flash                      +
;----------------------------------------------------
  NOP
  NOP
  NOP
load_prog_end:
;****************************************************
;+   end download module for flash HND type 2       +
;****************************************************
  .ENDM
